@@ -6,7 +6,6 @@
 #include "BZip2Update.h"
 
 #include "Common/Defs.h"
-#include "Common/String.h"
 
 #include "Windows/PropVariant.h"
 
@@ -50,20 +49,19 @@ STDMETHODIMP CHandler::UpdateItems(ISequentialOutStream *outStream, UInt32 numIt
   UInt32 indexInArchive;
   if (!updateCallback)
     return E_FAIL;
-  RINOK(updateCallback->GetUpdateItemInfo(0,
-      &newData, &newProperties, &indexInArchive));
+  RINOK(updateCallback->GetUpdateItemInfo(0,&newData, &newProperties, &indexInArchive));
  
   if (IntToBool(newProperties))
   {
     {
-      NCOM::CPropVariant propVariant;
-      RINOK(updateCallback->GetProperty(0, kpidIsFolder, &propVariant));
-      if (propVariant.vt == VT_BOOL)
+      NCOM::CPropVariant prop;
+      RINOK(updateCallback->GetProperty(0, kpidIsDir, &prop));
+      if (prop.vt == VT_BOOL)
       {
-        if (propVariant.boolVal != VARIANT_FALSE)
+        if (prop.boolVal != VARIANT_FALSE)
           return E_INVALIDARG;
       }
-      else if (propVariant.vt != VT_EMPTY)
+      else if (prop.vt != VT_EMPTY)
         return E_INVALIDARG;
     }
   }
@@ -72,30 +70,30 @@ STDMETHODIMP CHandler::UpdateItems(ISequentialOutStream *outStream, UInt32 numIt
   {
     UInt64 size;
     {
-      NCOM::CPropVariant propVariant;
-      RINOK(updateCallback->GetProperty(0, kpidSize, &propVariant));
-      if (propVariant.vt != VT_UI8)
+      NCOM::CPropVariant prop;
+      RINOK(updateCallback->GetProperty(0, kpidSize, &prop));
+      if (prop.vt != VT_UI8)
         return E_INVALIDARG;
-      size = propVariant.uhVal.QuadPart;
+      size = prop.uhVal.QuadPart;
     }
   
     UInt32 dicSize = _dicSize;
     if (dicSize == 0xFFFFFFFF)
-      dicSize = (_level >= 5 ? kDicSizeX5 : 
-                (_level >= 3 ? kDicSizeX3 : 
+      dicSize = (_level >= 5 ? kDicSizeX5 :
+                (_level >= 3 ? kDicSizeX3 :
                                kDicSizeX1));
 
     UInt32 numPasses = _numPasses;
     if (numPasses == 0xFFFFFFFF)
-      numPasses = (_level >= 9 ? kNumPassesX9 : 
-                  (_level >= 7 ? kNumPassesX7 : 
+      numPasses = (_level >= 9 ? kNumPassesX9 :
+                  (_level >= 7 ? kNumPassesX7 :
                                  kNumPassesX1));
 
     return UpdateArchive(
         EXTERNAL_CODECS_VARS
-        size, outStream, 0, dicSize, numPasses, 
+        size, outStream, 0, dicSize, numPasses,
         #ifdef COMPRESS_MT
-        _numThreads, 
+        _numThreads,
         #endif
         updateCallback);
   }
@@ -153,6 +151,6 @@ STDMETHODIMP CHandler::SetProperties(const wchar_t **names, const PROPVARIANT *v
     return E_INVALIDARG;
   }
   return S_OK;
-}  
+}
 
 }}
