@@ -30,23 +30,20 @@ using namespace NSynchronization;
 namespace NArchive {
 namespace NZip {
 
-class CCriticalSectionLock2
-{
-  CCriticalSection *_object;
-  void Unlock()  { if (_object != 0) _object->Leave(); }
-public:
-  CCriticalSectionLock2(): _object(0) {}
-  void Set(CCriticalSection &object) { _object = &object; _object->Enter(); } 
-  ~CCriticalSectionLock2() { Unlock(); }
-};
+static const Byte kHostOS =
+  #ifdef _WIN32
+  NFileHeader::NHostOS::kFAT;
+  #else
+  NFileHeader::NHostOS::kUnix;
+  #endif
 
-static const Byte kMadeByHostOS = NFileHeader::NHostOS::kFAT;
-static const Byte kExtractHostOS = NFileHeader::NHostOS::kFAT;
+static const Byte kMadeByHostOS = kHostOS;
+static const Byte kExtractHostOS = kHostOS;
 
 static const Byte kMethodForDirectory = NFileHeader::NCompressionMethod::kStored;
 static const Byte kExtractVersionForDirectory = NFileHeader::NCompressionMethod::kStoreExtractVersion;
 
-static HRESULT CopyBlockToArchive(ISequentialInStream *inStream, 
+static HRESULT CopyBlockToArchive(ISequentialInStream *inStream,
     COutArchive &outArchive, ICompressProgressInfo *progress)
 {
   CMyComPtr<ISequentialOutStream> outStream;
@@ -55,7 +52,7 @@ static HRESULT CopyBlockToArchive(ISequentialInStream *inStream,
   return copyCoder->Code(inStream, outStream, NULL, NULL, progress);
 }
 
-static HRESULT WriteRange(IInStream *inStream, COutArchive &outArchive, 
+static HRESULT WriteRange(IInStream *inStream, COutArchive &outArchive,
     const CUpdateRange &range, ICompressProgressInfo *progress)
 {
   UInt64 position;
@@ -71,23 +68,30 @@ static HRESULT WriteRange(IInStream *inStream, COutArchive &outArchive,
 }
 
 static void SetFileHeader(
-    COutArchive &archive, 
+    COutArchive &archive,
     const CCompressionMethodMode &options,
-    const CUpdateItem &updateItem, 
+    const CUpdateItem &ui,
     CItem &item)
 {
-  item.UnPackSize = updateItem.Size;
-  bool isDirectory;
+  item.UnPackSize = ui.Size;
+  bool isDir;
 
-  if (updateItem.NewProperties)
+  item.ClearFlags();
+
+  if (ui.NewProperties)
   {
-    isDirectory = updateItem.IsDirectory;
-    item.Name = updateItem.Name; 
-    item.ExternalAttributes = updateItem.Attributes;
-    item.Time = updateItem.Time;
+    isDir = ui.IsDir;
+    item.Name = ui.Name;
+    item.SetUtf8(ui.IsUtf8);
+    item.ExternalAttributes = ui.Attributes;
+    item.Time = ui.Time;
+    item.NtfsMTime = ui.NtfsMTime;
+    item.NtfsATime = ui.NtfsATime;
+    item.NtfsCTime = ui.NtfsCTime;
+    item.NtfsTimeIsDefined = ui.NtfsTimeIsDefined;
   }
   else
-    isDirectory = item.IsDirectory();
+    isDir = item.IsDir();
 
   item.LocalHeaderPosition = archive.GetCurrentPosition();
   item.MadeByVersion.HostOS = kMadeByHostOS;
@@ -96,9 +100,8 @@ static void SetFileHeader(
   item.ExtractVersion.HostOS = kExtractHostOS;
 
   item.InternalAttributes = 0; // test it
-  item.ClearFlags();
-  item.SetEncrypted(!isDirectory && options.PasswordIsDefined);
-  if (isDirectory)
+  item.SetEncrypted(!isDir && options.PasswordIsDefined);
+  if (isDir)
   {
     item.ExtractVersion.Version = kExtractVersionForDirectory;
     item.CompressionMethod = kMethodForDirectory;
@@ -107,7 +110,7 @@ static void SetFileHeader(
   }
 }
 
-static void SetItemInfoFromCompressingResult(const CCompressingResult &compressingResult, 
+static void SetItemInfoFromCompressingResult(const CCompressingResult &compressingResult,
     bool isAesMode, Byte aesKeyMode, CItem &item)
 {
   item.ExtractVersion.Version = compressingResult.ExtractVersion;
@@ -135,7 +138,7 @@ static void SetItemInfoFromCompressingResult(const CCompressingResult &compressi
 
 #ifdef COMPRESS_MT
 
-static DWORD WINAPI CoderThread(void *threadCoderInfo);
+static THREAD_FUNC_DECL CoderThread(void *threadCoderInfo);
 
 struct CThreadInfo
 {
@@ -145,8 +148,8 @@ struct CThreadInfo
   #endif
 
   NWindows::CThread Thread;
-  CAutoResetEvent *CompressEvent;
-  CAutoResetEvent *CompressionCompletedEvent;
+  NWindows::NSynchronization::CAutoResetEvent CompressEvent;
+  NWindows::NSynchronization::CAutoResetEventWFMO CompressionCompletedEvent;
   bool ExitThread;
 
   CMtCompressProgress *ProgressSpec;
@@ -164,21 +167,18 @@ struct CThreadInfo
   UInt32 UpdateIndex;
 
   CThreadInfo(const CCompressionMethodMode &options):
-      CompressEvent(NULL), 
-      CompressionCompletedEvent(NULL),
       ExitThread(false),
       ProgressSpec(0),
       OutStreamSpec(0),
       Coder(options)
   {}
   
-  void CreateEvents()
+  HRESULT CreateEvents(CSynchro *sync)
   {
-    CompressEvent = new CAutoResetEvent(false);
-    CompressionCompletedEvent = new CAutoResetEvent(false);
+    RINOK(CompressEvent.CreateIfNotCreated());
+    return CompressionCompletedEvent.CreateIfNotCreated(sync);
   }
-  bool CreateThread() { return Thread.Create(CoderThread, this); }
-  ~CThreadInfo();
+  HRes CreateThread() { return Thread.Create(CoderThread, this); }
 
   void WaitAndCode();
   void StopWaitClose()
@@ -186,41 +186,33 @@ struct CThreadInfo
     ExitThread = true;
     if (OutStreamSpec != 0)
       OutStreamSpec->StopWriting(E_ABORT);
-    if (CompressEvent != NULL)
-      CompressEvent->Set();
+    if (CompressEvent.IsCreated())
+      CompressEvent.Set();
     Thread.Wait();
     Thread.Close();
   }
 
 };
 
-CThreadInfo::~CThreadInfo()
-{
-  if (CompressEvent != NULL)
-    delete CompressEvent;
-  if (CompressionCompletedEvent != NULL)
-    delete CompressionCompletedEvent;
-}
-
 void CThreadInfo::WaitAndCode()
 {
   for (;;)
   {
-    CompressEvent->Lock();
+    CompressEvent.Lock();
     if (ExitThread)
       return;
     Result = Coder.Compress(
         #ifdef EXTERNAL_CODECS
-        _codecsInfo, _externalCodecs, 
+        _codecsInfo, _externalCodecs,
         #endif
         InStream, OutStream, Progress, CompressingResult);
     if (Result == S_OK && Progress)
       Result = Progress->SetRatioInfo(&CompressingResult.UnpackSize, &CompressingResult.PackSize);
-    CompressionCompletedEvent->Set();
+    CompressionCompletedEvent.Set();
   }
 }
 
-static DWORD WINAPI CoderThread(void *threadCoderInfo)
+static THREAD_FUNC_DECL CoderThread(void *threadCoderInfo)
 {
   ((CThreadInfo *)threadCoderInfo)->WaitAndCode();
   return 0;
@@ -258,15 +250,99 @@ public:
   }
 };
 
+class CMtProgressMixer2:
+  public ICompressProgressInfo,
+  public CMyUnknownImp
+{
+  UInt64 ProgressOffset;
+  UInt64 InSizes[2];
+  UInt64 OutSizes[2];
+  CMyComPtr<IProgress> Progress;
+  CMyComPtr<ICompressProgressInfo> RatioProgress;
+  bool _inSizeIsMain;
+public:
+  NWindows::NSynchronization::CCriticalSection CriticalSection;
+  MY_UNKNOWN_IMP
+  void Create(IProgress *progress, bool inSizeIsMain);
+  void SetProgressOffset(UInt64 progressOffset);
+  HRESULT SetRatioInfo(int index, const UInt64 *inSize, const UInt64 *outSize);
+  STDMETHOD(SetRatioInfo)(const UInt64 *inSize, const UInt64 *outSize);
+};
+
+void CMtProgressMixer2::Create(IProgress *progress, bool inSizeIsMain)
+{
+  Progress = progress;
+  Progress.QueryInterface(IID_ICompressProgressInfo, &RatioProgress);
+  _inSizeIsMain = inSizeIsMain;
+  ProgressOffset = InSizes[0] = InSizes[1] = OutSizes[0] = OutSizes[1] = 0;
+}
+
+void CMtProgressMixer2::SetProgressOffset(UInt64 progressOffset)
+{
+  CriticalSection.Enter();
+  InSizes[1] = OutSizes[1] = 0;
+  ProgressOffset = progressOffset;
+  CriticalSection.Leave();
+}
+
+HRESULT CMtProgressMixer2::SetRatioInfo(int index, const UInt64 *inSize, const UInt64 *outSize)
+{
+  NWindows::NSynchronization::CCriticalSectionLock lock(CriticalSection);
+  if (index == 0 && RatioProgress)
+  {
+    RINOK(RatioProgress->SetRatioInfo(inSize, outSize));
+  }
+  if (inSize != 0)
+    InSizes[index] = *inSize;
+  if (outSize != 0)
+    OutSizes[index] = *outSize;
+  UInt64 v = ProgressOffset + (_inSizeIsMain  ?
+      (InSizes[0] + InSizes[1]) :
+      (OutSizes[0] + OutSizes[1]));
+  return Progress->SetCompleted(&v);
+}
+
+STDMETHODIMP CMtProgressMixer2::SetRatioInfo(const UInt64 *inSize, const UInt64 *outSize)
+{
+  return SetRatioInfo(0, inSize, outSize);
+}
+
+class CMtProgressMixer:
+  public ICompressProgressInfo,
+  public CMyUnknownImp
+{
+public:
+  CMtProgressMixer2 *Mixer2;
+  CMyComPtr<ICompressProgressInfo> RatioProgress;
+  void Create(IProgress *progress, bool inSizeIsMain);
+  MY_UNKNOWN_IMP
+  STDMETHOD(SetRatioInfo)(const UInt64 *inSize, const UInt64 *outSize);
+};
+
+void CMtProgressMixer::Create(IProgress *progress, bool inSizeIsMain)
+{
+  Mixer2 = new CMtProgressMixer2;
+  RatioProgress = Mixer2;
+  Mixer2->Create(progress, inSizeIsMain);
+}
+
+STDMETHODIMP CMtProgressMixer::SetRatioInfo(const UInt64 *inSize, const UInt64 *outSize)
+{
+  return Mixer2->SetRatioInfo(1, inSize, outSize);
+}
+
+
 #endif
 
 
-static HRESULT UpdateItemOldData(COutArchive &archive, 
+static HRESULT UpdateItemOldData(COutArchive &archive,
     IInStream *inStream,
-    const CUpdateItem &updateItem, CItemEx &item, ICompressProgressInfo *progress, 
+    const CUpdateItem &ui, CItemEx &item,
+    /* bool izZip64, */
+    ICompressProgressInfo *progress,
     UInt64 &complexity)
 {
-  if (updateItem.NewProperties)
+  if (ui.NewProperties)
   {
     if (item.HasDescriptor())
       return E_NOTIMPL;
@@ -275,10 +351,16 @@ static HRESULT UpdateItemOldData(COutArchive &archive,
     // CUpdateRange range(item.GetLocalExtraPosition(), item.LocalExtraSize + item.PackSize);
     CUpdateRange range(item.GetDataPosition(), item.PackSize);
     
-    // item.ExternalAttributes = updateItem.Attributes;
+    // item.ExternalAttributes = ui.Attributes;
     // Test it
-    item.Name = updateItem.Name; 
-    item.Time = updateItem.Time;
+    item.Name = ui.Name;
+    item.SetUtf8(ui.IsUtf8);
+    item.Time = ui.Time;
+    item.NtfsMTime = ui.NtfsMTime;
+    item.NtfsATime = ui.NtfsATime;
+    item.NtfsCTime = ui.NtfsCTime;
+    item.NtfsTimeIsDefined = ui.NtfsTimeIsDefined;
+
     item.CentralExtra.RemoveUnknownSubBlocks();
     item.LocalExtra.RemoveUnknownSubBlocks();
     
@@ -294,7 +376,7 @@ static HRESULT UpdateItemOldData(COutArchive &archive,
     CUpdateRange range(item.LocalHeaderPosition, item.GetLocalFullSize());
     
     // set new header position
-    item.LocalHeaderPosition = archive.GetCurrentPosition(); 
+    item.LocalHeaderPosition = archive.GetCurrentPosition();
     
     RINOK(WriteRange(inStream, archive, range, progress));
     complexity += range.Size;
@@ -303,91 +385,93 @@ static HRESULT UpdateItemOldData(COutArchive &archive,
   return S_OK;
 }
 
-static HRESULT WriteDirHeader(COutArchive &archive, const CCompressionMethodMode *options,
-    const CUpdateItem &updateItem, CItemEx &item)
+static void WriteDirHeader(COutArchive &archive, const CCompressionMethodMode *options,
+    const CUpdateItem &ui, CItemEx &item)
 {
-  SetFileHeader(archive, *options, updateItem, item);
-  archive.PrepareWriteCompressedData((UInt16)item.Name.Length(), updateItem.Size, options->IsAesMode);
-  return archive.WriteLocalHeader(item);
+  SetFileHeader(archive, *options, ui, item);
+  archive.PrepareWriteCompressedData((UInt16)item.Name.Length(), ui.Size, options->IsAesMode);
+  archive.WriteLocalHeader(item);
 }
 
 static HRESULT Update2St(
     DECL_EXTERNAL_CODECS_LOC_VARS
-    COutArchive &archive, 
+    COutArchive &archive,
     CInArchive *inArchive,
     IInStream *inStream,
     const CObjectVector<CItemEx> &inputItems,
     const CObjectVector<CUpdateItem> &updateItems,
-    const CCompressionMethodMode *options, 
+    const CCompressionMethodMode *options,
     const CByteBuffer &comment,
     IArchiveUpdateCallback *updateCallback)
 {
-  CLocalProgress *localProgressSpec = new CLocalProgress;
-  CMyComPtr<ICompressProgressInfo> localProgress = localProgressSpec;
-  localProgressSpec->Init(updateCallback, true);
-
-  CLocalCompressProgressInfo *localCompressProgressSpec = new CLocalCompressProgressInfo;
-  CMyComPtr<ICompressProgressInfo> compressProgress = localCompressProgressSpec;
+  CLocalProgress *lps = new CLocalProgress;
+  CMyComPtr<ICompressProgressInfo> progress = lps;
+  lps->Init(updateCallback, true);
 
   CAddCommon compressor(*options);
   
   CObjectVector<CItem> items;
-  UInt64 complexity = 0;
+  UInt64 unpackSizeTotal = 0, packSizeTotal = 0;
 
   for (int itemIndex = 0; itemIndex < updateItems.Size(); itemIndex++)
   {
-    RINOK(updateCallback->SetCompleted(&complexity));
-    const CUpdateItem &updateItem = updateItems[itemIndex];
+    lps->InSize = unpackSizeTotal;
+    lps->OutSize = packSizeTotal;
+    RINOK(lps->SetCur());
+    const CUpdateItem &ui = updateItems[itemIndex];
     CItemEx item;
-    if (!updateItem.NewProperties || !updateItem.NewData)
+    if (!ui.NewProperties || !ui.NewData)
     {
-      item = inputItems[updateItem.IndexInArchive];
+      item = inputItems[ui.IndexInArchive];
       if (inArchive->ReadLocalItemAfterCdItemFull(item) != S_OK)
         return E_NOTIMPL;
     }
 
-    if (updateItem.NewData)
+    if (ui.NewData)
     {
-      bool isDirectory = ((updateItem.NewProperties) ? updateItem.IsDirectory : item.IsDirectory());
-      if (isDirectory)
+      bool isDir = ((ui.NewProperties) ? ui.IsDir : item.IsDir());
+      if (isDir)
       {
-        RINOK(WriteDirHeader(archive, options, updateItem, item));
+        WriteDirHeader(archive, options, ui, item);
       }
       else
       {
         CMyComPtr<ISequentialInStream> fileInStream;
-        HRESULT res = updateCallback->GetStream(updateItem.IndexInClient, &fileInStream);
+        HRESULT res = updateCallback->GetStream(ui.IndexInClient, &fileInStream);
         if (res == S_FALSE)
         {
-          complexity += updateItem.Size + NFileHeader::kLocalBlockSize;
+          lps->ProgressOffset += ui.Size;
           RINOK(updateCallback->SetOperationResult(NArchive::NUpdate::NOperationResult::kOK));
           continue;
         }
         RINOK(res);
 
         // file Size can be 64-bit !!!
-        SetFileHeader(archive, *options, updateItem, item);
-        archive.PrepareWriteCompressedData((UInt16)item.Name.Length(), updateItem.Size, options->IsAesMode);
+        SetFileHeader(archive, *options, ui, item);
+        archive.PrepareWriteCompressedData((UInt16)item.Name.Length(), ui.Size, options->IsAesMode);
         CCompressingResult compressingResult;
         CMyComPtr<IOutStream> outStream;
         archive.CreateStreamForCompressing(&outStream);
-        localCompressProgressSpec->Init(localProgress, &complexity, NULL);
         RINOK(compressor.Compress(
             EXTERNAL_CODECS_LOC_VARS
-            fileInStream, outStream, compressProgress, compressingResult));
+            fileInStream, outStream, progress, compressingResult));
         SetItemInfoFromCompressingResult(compressingResult, options->IsAesMode, options->AesKeyMode, item);
-        RINOK(archive.WriteLocalHeader(item));
+        archive.WriteLocalHeader(item);
         RINOK(updateCallback->SetOperationResult(NArchive::NUpdate::NOperationResult::kOK));
-        complexity += item.UnPackSize;
+        unpackSizeTotal += item.UnPackSize;
+        packSizeTotal += item.PackSize;
       }
     }
     else
     {
-      localCompressProgressSpec->Init(localProgress, &complexity, NULL);
-      RINOK(UpdateItemOldData(archive, inStream, updateItem, item, compressProgress, complexity));
+      UInt64 complexity = 0;
+      lps->SendRatio = false;
+      RINOK(UpdateItemOldData(archive, inStream, ui, item, progress, complexity));
+      lps->SendRatio = true;
+      lps->ProgressOffset += complexity;
     }
     items.Add(item);
-    complexity += NFileHeader::kLocalBlockSize;
+    lps->ProgressOffset += NFileHeader::kLocalBlockSize;
   }
   archive.WriteCentralDir(items, comment);
   return S_OK;
@@ -395,12 +479,12 @@ static HRESULT Update2St(
 
 static HRESULT Update2(
     DECL_EXTERNAL_CODECS_LOC_VARS
-    COutArchive &archive, 
+    COutArchive &archive,
     CInArchive *inArchive,
     IInStream *inStream,
     const CObjectVector<CItemEx> &inputItems,
     const CObjectVector<CUpdateItem> &updateItems,
-    const CCompressionMethodMode *options, 
+    const CCompressionMethodMode *options,
     const CByteBuffer &comment,
     IArchiveUpdateCallback *updateCallback)
 {
@@ -411,20 +495,20 @@ static HRESULT Update2(
   int i;
   for(i = 0; i < updateItems.Size(); i++)
   {
-    const CUpdateItem &updateItem = updateItems[i];
-    if (updateItem.NewData)
+    const CUpdateItem &ui = updateItems[i];
+    if (ui.NewData)
     {
-      complexity += updateItem.Size;
-      numBytesToCompress += updateItem.Size;
+      complexity += ui.Size;
+      numBytesToCompress += ui.Size;
       numFilesToCompress++;
       /*
-      if (updateItem.Commented)
-        complexity += updateItem.CommentRange.Size;
+      if (ui.Commented)
+        complexity += ui.CommentRange.Size;
       */
     }
     else
     {
-      CItemEx inputItem = inputItems[updateItem.IndexInArchive];
+      CItemEx inputItem = inputItems[ui.IndexInArchive];
       if (inArchive->ReadLocalItemAfterCdItemFull(inputItem) != S_OK)
         return E_NOTIMPL;
       complexity += inputItem.GetLocalFullSize();
@@ -484,6 +568,14 @@ static HRESULT Update2(
       if (numThreads <= 1)
         mtMode = false;
     }
+    if (method == NFileHeader::NCompressionMethod::kLZMA)
+    {
+      UInt32 numLZMAThreads = (options->Algo > 0 ? 2 : 1);
+      numThreads /= numLZMAThreads;
+      options2.NumThreads = numLZMAThreads;
+      if (numThreads <= 1)
+        mtMode = false;
+    }
   }
 
   if (!mtMode)
@@ -496,19 +588,22 @@ static HRESULT Update2(
 
   #ifdef COMPRESS_MT
 
+  // Warning : before memManager, threads and compressingCompletedEvents
+  // in order to have a "good" order for the destructor
+  NWindows::NSynchronization::CSynchro synchroForCompressingCompletedEvents;
+  synchroForCompressingCompletedEvents.Create();
+  NWindows::NSynchronization::CSynchro synchroForOutStreamSpec;
+  synchroForOutStreamSpec.Create();
+  
+
   CObjectVector<CItem> items;
 
-  CLocalProgress *localProgressSpec = new CLocalProgress;
-  CMyComPtr<ICompressProgressInfo> localProgress = localProgressSpec;
-  localProgressSpec->Init(updateCallback, true);
+  CMtProgressMixer *mtProgressMixerSpec = new CMtProgressMixer;
+  CMyComPtr<ICompressProgressInfo> progress = mtProgressMixerSpec;
+  mtProgressMixerSpec->Create(updateCallback, true);
 
   CMtCompressProgressMixer mtCompressProgressMixer;
-  mtCompressProgressMixer.Init(numThreads + 1, localProgress); 
-
-  // we need one item for main stream
-  CMtCompressProgress *progressMainSpec  = new CMtCompressProgress();
-  CMyComPtr<ICompressProgressInfo> progressMain = progressMainSpec;
-  progressMainSpec->Init(&mtCompressProgressMixer, (int)numThreads);
+  mtCompressProgressMixer.Init(numThreads, mtProgressMixerSpec->RatioProgress);
 
   CMemBlockManagerMt memManager(kBlockSize);
   CMemRefs refs(&memManager);
@@ -518,8 +613,7 @@ static HRESULT Update2(
   CRecordVector<int> threadIndices;  // list threads in order of updateItems
 
   {
-    if (!memManager.AllocateSpaceAlways((size_t)numThreads * (kMemPerThread / kBlockSize)))
-      return E_OUTOFMEMORY;
+    RINOK(memManager.AllocateSpaceAlways(&synchroForOutStreamSpec,(size_t)numThreads * (kMemPerThread / kBlockSize)));
     for(i = 0; i < updateItems.Size(); i++)
       refs.Refs.Add(CMemBlocks2());
 
@@ -534,15 +628,15 @@ static HRESULT Update2(
       threadInfo._codecsInfo = codecsInfo;
       threadInfo._externalCodecs = externalCodecs;
       #endif
-      threadInfo.CreateEvents();
+      RINOK(threadInfo.CreateEvents(&synchroForCompressingCompletedEvents));
       threadInfo.OutStreamSpec = new COutMemStream(&memManager);
+      RINOK(threadInfo.OutStreamSpec->CreateEvents(&synchroForOutStreamSpec));
       threadInfo.OutStream = threadInfo.OutStreamSpec;
       threadInfo.IsFree = true;
       threadInfo.ProgressSpec = new CMtCompressProgress();
       threadInfo.Progress = threadInfo.ProgressSpec;
       threadInfo.ProgressSpec->Init(&mtCompressProgressMixer, (int)i);
-      if (!threadInfo.CreateThread())
-        return ::GetLastError();
+      RINOK(threadInfo.CreateThread());
     }
   }
   int mtItemIndex = 0;
@@ -554,31 +648,32 @@ static HRESULT Update2(
   {
     if ((UInt32)threadIndices.Size() < numThreads && mtItemIndex < updateItems.Size())
     {
-      const CUpdateItem &updateItem = updateItems[mtItemIndex++];
-      if (!updateItem.NewData)
+      const CUpdateItem &ui = updateItems[mtItemIndex++];
+      if (!ui.NewData)
         continue;
       CItemEx item;
-      if (updateItem.NewProperties)
+      if (ui.NewProperties)
       {
-        if (updateItem.IsDirectory)
+        if (ui.IsDir)
           continue;
       }
       else
       {
-        item = inputItems[updateItem.IndexInArchive];
+        item = inputItems[ui.IndexInArchive];
         if (inArchive->ReadLocalItemAfterCdItemFull(item) != S_OK)
           return E_NOTIMPL;
-        if (item.IsDirectory())
+        if (item.IsDir())
           continue;
       }
       CMyComPtr<ISequentialInStream> fileInStream;
       {
-        NWindows::NSynchronization::CCriticalSectionLock lock(mtCompressProgressMixer.CriticalSection);
-        HRESULT res = updateCallback->GetStream(updateItem.IndexInClient, &fileInStream);
+        NWindows::NSynchronization::CCriticalSectionLock lock(mtProgressMixerSpec->Mixer2->CriticalSection);
+        HRESULT res = updateCallback->GetStream(ui.IndexInClient, &fileInStream);
         if (res == S_FALSE)
         {
-          complexity += updateItem.Size;
-          complexity++;
+          complexity += ui.Size;
+          complexity += NFileHeader::kLocalBlockSize;
+          mtProgressMixerSpec->Mixer2->SetProgressOffset(complexity);
           RINOK(updateCallback->SetOperationResult(NArchive::NUpdate::NOperationResult::kOK));
           refs.Refs[mtItemIndex - 1].Skip = true;
           continue;
@@ -595,16 +690,16 @@ static HRESULT Update2(
           threadInfo.IsFree = false;
           threadInfo.InStream = fileInStream;
 
-          // !!!!! we must release ref before sending event 
+          // !!!!! we must release ref before sending event
           // BUG was here in v4.43 and v4.44. It could change ref counter in two threads in same time
-          fileInStream.Release(); 
+          fileInStream.Release();
 
           threadInfo.OutStreamSpec->Init();
           threadInfo.ProgressSpec->Reinit();
-          threadInfo.CompressEvent->Set();
+          threadInfo.CompressEvent.Set();
           threadInfo.UpdateIndex = mtItemIndex - 1;
 
-          compressingCompletedEvents.Add(*threadInfo.CompressionCompletedEvent);
+          compressingCompletedEvents.Add(threadInfo.CompressionCompletedEvent);
           threadIndices.Add(i);
           break;
         }
@@ -618,31 +713,31 @@ static HRESULT Update2(
       continue;
     }
 
-    const CUpdateItem &updateItem = updateItems[itemIndex];
+    const CUpdateItem &ui = updateItems[itemIndex];
 
     CItemEx item;
-    if (!updateItem.NewProperties || !updateItem.NewData)
+    if (!ui.NewProperties || !ui.NewData)
     {
-      item = inputItems[updateItem.IndexInArchive];
+      item = inputItems[ui.IndexInArchive];
       if (inArchive->ReadLocalItemAfterCdItemFull(item) != S_OK)
         return E_NOTIMPL;
     }
 
-    if (updateItem.NewData)
+    if (ui.NewData)
     {
-      bool isDirectory = ((updateItem.NewProperties) ? updateItem.IsDirectory : item.IsDirectory());
-      if (isDirectory)
+      bool isDir = ((ui.NewProperties) ? ui.IsDir : item.IsDir());
+      if (isDir)
       {
-        RINOK(WriteDirHeader(archive, options, updateItem, item));
+        WriteDirHeader(archive, options, ui, item);
       }
       else
       {
         if (lastRealStreamItemIndex < itemIndex)
         {
           lastRealStreamItemIndex = itemIndex;
-          SetFileHeader(archive, *options, updateItem, item);
+          SetFileHeader(archive, *options, ui, item);
           // file Size can be 64-bit !!!
-          archive.PrepareWriteCompressedData((UInt16)item.Name.Length(), updateItem.Size, options->IsAesMode);
+          archive.PrepareWriteCompressedData((UInt16)item.Name.Length(), ui.Size, options->IsAesMode);
         }
 
         CMemBlocks2 &memRef = refs.Refs[itemIndex];
@@ -653,9 +748,8 @@ static HRESULT Update2(
           memRef.WriteToStream(memManager.GetBlockSize(), outStream);
           SetItemInfoFromCompressingResult(memRef.CompressingResult,
               options->IsAesMode, options->AesKeyMode, item);
-          SetFileHeader(archive, *options, updateItem, item);
-          RINOK(archive.WriteLocalHeader(item));
-          complexity += item.UnPackSize;
+          SetFileHeader(archive, *options, ui, item);
+          archive.WriteLocalHeader(item);
           // RINOK(updateCallback->SetOperationResult(NArchive::NUpdate::NOperationResult::kOK));
           memRef.FreeOpt(&memManager);
         }
@@ -672,7 +766,7 @@ static HRESULT Update2(
             }
           }
 
-          DWORD result = ::WaitForMultipleObjects(compressingCompletedEvents.Size(), 
+          DWORD result = ::WaitForMultipleObjects(compressingCompletedEvents.Size(),
               &compressingCompletedEvents.Front(), FALSE, INFINITE);
           int t = (int)(result - WAIT_OBJECT_0);
           CThreadInfo &threadInfo = threads.Threads[threadIndices[t]];
@@ -685,11 +779,10 @@ static HRESULT Update2(
           {
             RINOK(threadInfo.OutStreamSpec->WriteToRealStream());
             threadInfo.OutStreamSpec->ReleaseOutStream();
-            SetItemInfoFromCompressingResult(threadInfo.CompressingResult, 
+            SetItemInfoFromCompressingResult(threadInfo.CompressingResult,
                 options->IsAesMode, options->AesKeyMode, item);
-            SetFileHeader(archive, *options, updateItem, item);
-            RINOK(archive.WriteLocalHeader(item));
-            complexity += item.UnPackSize;
+            SetFileHeader(archive, *options, ui, item);
+            archive.WriteLocalHeader(item);
           }
           else
           {
@@ -704,19 +797,19 @@ static HRESULT Update2(
     }
     else
     {
-      progressMainSpec->Reinit();
-      RINOK(UpdateItemOldData(archive, inStream, updateItem, item, progressMain, complexity));
+      RINOK(UpdateItemOldData(archive, inStream, ui, item, progress, complexity));
     }
     items.Add(item);
     complexity += NFileHeader::kLocalBlockSize;
+    mtProgressMixerSpec->Mixer2->SetProgressOffset(complexity);
     itemIndex++;
   }
   archive.WriteCentralDir(items, comment);
   return S_OK;
-  #endif  
+  #endif
 }
 
-HRESULT Update(    
+HRESULT Update(
     DECL_EXTERNAL_CODECS_LOC_VARS
     const CObjectVector<CItemEx> &inputItems,
     const CObjectVector<CUpdateItem> &updateItems,
@@ -755,9 +848,9 @@ HRESULT Update(
 
   return Update2(
       EXTERNAL_CODECS_LOC_VARS
-      outArchive, inArchive, inStream, 
-      inputItems, updateItems, 
-      compressionMethodMode, 
+      outArchive, inArchive, inStream,
+      inputItems, updateItems,
+      compressionMethodMode,
       archiveInfo.Comment, updateCallback);
 }
 
