@@ -2,22 +2,18 @@
 
 #include "StdAfx.h"
 
-extern "C" 
-{ 
+extern "C"
+{
 #include "../../../../C/Alloc.h"
+#include "../../../../C/BwtSort.h"
+#include "../../../../C/HuffEnc.h"
 }
 
 #include "BZip2Encoder.h"
 
-#include "../BWT/BlockSort.h"
 #include "../BWT/Mtf8.h"
 #include "BZip2CRC.h"
 
-extern "C"
-{
-  #include "../../../../C/Compress/Huffman/HuffmanEncode.h"
-}
-
 namespace NCompress {
 namespace NBZip2 {
 
@@ -61,12 +57,15 @@ static THREAD_FUNC_DECL MFThread(void *threadCoderInfo)
   return ((CThreadInfo *)threadCoderInfo)->ThreadFunc();
 }
 
-HRes CThreadInfo::Create()
+#define RINOK_THREAD(x) { WRes __result_ = (x); if(__result_ != 0) return __result_; }
+
+HRESULT CThreadInfo::Create()
 {
-  RINOK(StreamWasFinishedEvent.Create());
-  RINOK(WaitingWasStartedEvent.Create());
-  RINOK(CanWriteEvent.Create());
-  return Thread.Create(MFThread, this);
+  RINOK_THREAD(StreamWasFinishedEvent.Create());
+  RINOK_THREAD(WaitingWasStartedEvent.Create());
+  RINOK_THREAD(CanWriteEvent.Create());
+  RINOK_THREAD(Thread.Create(MFThread, this));
+  return S_OK;
 }
 
 void CThreadInfo::FinishStream(bool needLeave)
@@ -97,7 +96,7 @@ DWORD CThreadInfo::ThreadFunc()
     }
     HRESULT res = S_OK;
     bool needLeave = true;
-    try 
+    try
     {
       UInt32 blockSize = Encoder->ReadRleBlock(m_Block);
       m_PackSize = Encoder->m_InStream.GetProcessedSize();
@@ -128,7 +127,7 @@ DWORD CThreadInfo::ThreadFunc()
 #endif
 
 CEncoder::CEncoder():
-  NumPasses(1), 
+  NumPasses(1),
   m_OptimizeNumTables(false),
   m_BlockSizeMult(kBlockSizeMultMax)
 {
@@ -145,14 +144,14 @@ CEncoder::~CEncoder()
   Free();
 }
 
-HRes CEncoder::Create()
+HRESULT CEncoder::Create()
 {
-  RINOK(CanProcessEvent.CreateIfNotCreated());
-  RINOK(CanStartWaitingEvent.CreateIfNotCreated());
+  RINOK_THREAD(CanProcessEvent.CreateIfNotCreated());
+  RINOK_THREAD(CanStartWaitingEvent.CreateIfNotCreated());
   if (ThreadsInfo != 0 && m_NumThreadsPrev == NumThreads)
     return S_OK;
-  try 
-  { 
+  try
+  {
     Free();
     MtMode = (NumThreads > 1);
     m_NumThreadsPrev = NumThreads;
@@ -167,12 +166,12 @@ HRes CEncoder::Create()
     ti.Encoder = this;
     if (MtMode)
     {
-      HRes res = ti.Create();
+      HRESULT res = ti.Create();
       if (res != S_OK)
       {
         NumThreads = t;
         Free();
-        return res; 
+        return res;
       }
     }
   }
@@ -240,8 +239,8 @@ void CThreadInfo::WriteBits2(UInt32 value, UInt32 numBits)
   { m_OutStreamCurrent->WriteBits(value, numBits); }
 void CThreadInfo::WriteByte2(Byte b) { WriteBits2(b , 8); }
 void CThreadInfo::WriteBit2(bool v) { WriteBits2((v ? 1 : 0), 1); }
-void CThreadInfo::WriteCRC2(UInt32 v) 
-{ 
+void CThreadInfo::WriteCRC2(UInt32 v)
+{
   for (int i = 0; i < 4; i++)
     WriteByte2(((Byte)(v >> (24 - i * 8))));
 }
@@ -250,8 +249,8 @@ void CEncoder::WriteBits(UInt32 value, UInt32 numBits)
   { m_OutStream.WriteBits(value, numBits); }
 void CEncoder::WriteByte(Byte b) { WriteBits(b , 8); }
 void CEncoder::WriteBit(bool v) { WriteBits((v ? 1 : 0), 1); }
-void CEncoder::WriteCRC(UInt32 v) 
-{ 
+void CEncoder::WriteCRC(UInt32 v)
+{
   for (int i = 0; i < 4; i++)
     WriteByte(((Byte)(v >> (24 - i * 8))));
 }
@@ -275,21 +274,21 @@ void CThreadInfo::EncodeBlock(const Byte *block, UInt32 blockSize)
     bool inUse[256];
     bool inUse16[16];
     UInt32 i;
-    for (i = 0; i < 256; i++) 
+    for (i = 0; i < 256; i++)
       inUse[i] = false;
-    for (i = 0; i < 16; i++) 
+    for (i = 0; i < 16; i++)
       inUse16[i] = false;
     for (i = 0; i < blockSize; i++)
       inUse[block[i]] = true;
-    for (i = 0; i < 256; i++) 
+    for (i = 0; i < 256; i++)
       if (inUse[i])
       {
         inUse16[i >> 4] = true;
         mtf.Buffer[numInUse++] = (Byte)i;
       }
-    for (i = 0; i < 16; i++) 
+    for (i = 0; i < 16; i++)
       WriteBit2(inUse16[i]);
-    for (i = 0; i < 256; i++) 
+    for (i = 0; i < 256; i++)
       if (inUse16[i >> 4])
         WriteBit2(inUse[i]);
   }
@@ -308,7 +307,7 @@ void CThreadInfo::EncodeBlock(const Byte *block, UInt32 blockSize)
     UInt32 i = 0;
     const UInt32 *bsIndex = m_BlockSorterIndex;
     block--;
-    do 
+    do
     {
       int pos = mtf.FindAndMove(block[bsIndex[i]]);
       if (pos == 0)
@@ -377,10 +376,10 @@ void CThreadInfo::EncodeBlock(const Byte *block, UInt32 blockSize)
     }
     else
     {
-      if (numSymbols < 200)  numTables = 2; 
-      else if (numSymbols < 600) numTables = 3; 
-      else if (numSymbols < 1200) numTables = 4; 
-      else if (numSymbols < 2400) numTables = 5; 
+      if (numSymbols < 200)  numTables = 2;
+      else if (numSymbols < 600) numTables = 3;
+      else if (numSymbols < 1200) numTables = 4;
+      else if (numSymbols < 2400) numTables = 5;
       else numTables = 6;
     }
 
@@ -398,10 +397,10 @@ void CThreadInfo::EncodeBlock(const Byte *block, UInt32 blockSize)
         UInt32 tFreq = remFreq / t;
         int ge = gs;
         UInt32 aFreq = 0;
-        while (aFreq < tFreq) //  && ge < alphaSize) 
+        while (aFreq < tFreq) //  && ge < alphaSize)
           aFreq += symbolCounts[ge++];
         
-        if (ge - 1 > gs && t != numTables && t != 1 && (((numTables - t) & 1) == 1)) 
+        if (ge - 1 > gs && t != numTables && t != 1 && (((numTables - t) & 1) == 1))
           aFreq -= symbolCounts[--ge];
         
         Byte *lens = Lens[t - 1];
@@ -428,7 +427,7 @@ void CThreadInfo::EncodeBlock(const Byte *block, UInt32 blockSize)
       {
         UInt32 mtfPos = 0;
         UInt32 g = 0;
-        do 
+        do
         {
           UInt32 symbols[kGroupSize];
           int i = 0;
@@ -524,7 +523,7 @@ void CThreadInfo::EncodeBlock(const Byte *block, UInt32 blockSize)
               WriteBit2(false);
               len++;
             }
-            else 
+            else
             {
               WriteBit2(true);
               len--;
@@ -548,7 +547,7 @@ void CThreadInfo::EncodeBlock(const Byte *block, UInt32 blockSize)
         UInt32 symbol = mtfs[mtfPos++];
         if (symbol >= 0xFF)
           symbol += mtfs[mtfPos++];
-        if (groupSize == 0) 
+        if (groupSize == 0)
         {
           groupSize = kGroupSize;
           int t = m_Selectors[groupIndex++];
@@ -588,7 +587,7 @@ UInt32 CThreadInfo::EncodeBlockWithHeaders(const Byte *block, UInt32 blockSize)
   int numReps = 0;
   Byte prevByte = block[0];
   UInt32 i = 0;
-  do 
+  do
   {
     Byte b = block[i];
     if (numReps == kRleModeRepSize)
@@ -628,7 +627,7 @@ void CThreadInfo::EncodeBlock2(const Byte *block, UInt32 blockSize, UInt32 numPa
   {
     UInt32 blockSize0 = blockSize / 2;
     for (;(block[blockSize0] == block[blockSize0 - 1] ||
-          block[blockSize0 - 1] == block[blockSize0 - 2]) && 
+          block[blockSize0 - 1] == block[blockSize0 - 2]) &&
           blockSize0 < blockSize; blockSize0++);
     if (blockSize0 < blockSize)
     {
@@ -652,7 +651,7 @@ void CThreadInfo::EncodeBlock2(const Byte *block, UInt32 blockSize, UInt32 numPa
   {
     UInt32 size2 = endPos2 - startPos2;
     if (size2 < endPos - startPos)
-    { 
+    {
       UInt32 numBytes = m_OutStreamCurrent->GetBytePos() - startBytePos2;
       Byte *buffer = m_OutStreamCurrent->GetStream();
       for (UInt32 i = 0; i < numBytes; i++)
@@ -733,9 +732,12 @@ HRESULT CEncoder::CodeReal(ISequentialInStream *inStream,
   {
     #ifdef COMPRESS_BZIP2_MT
     CThreadInfo &ti = ThreadsInfo[t];
-    ti.StreamWasFinishedEvent.Reset();
-    ti.WaitingWasStartedEvent.Reset();
-    ti.CanWriteEvent.Reset();
+    if (MtMode)
+    {
+      RINOK(ti.StreamWasFinishedEvent.Reset());
+      RINOK(ti.WaitingWasStartedEvent.Reset());
+      RINOK(ti.CanWriteEvent.Reset());
+    }
     #else
     CThreadInfo &ti = ThreadsInfo;
     ti.Encoder = this;
@@ -797,7 +799,7 @@ HRESULT CEncoder::CodeReal(ISequentialInStream *inStream,
   {
     for (;;)
     {
-      CThreadInfo &ti = 
+      CThreadInfo &ti =
       #ifdef COMPRESS_BZIP2_MT
       ThreadsInfo[0];
       #else
@@ -836,12 +838,12 @@ STDMETHODIMP CEncoder::Code(ISequentialInStream *inStream,
   catch(...) { return S_FALSE; }
 }
 
-HRESULT CEncoder::SetCoderProperties(const PROPID *propIDs, 
+HRESULT CEncoder::SetCoderProperties(const PROPID *propIDs,
     const PROPVARIANT *properties, UInt32 numProperties)
 {
   for(UInt32 i = 0; i < numProperties; i++)
   {
-    const PROPVARIANT &property = properties[i]; 
+    const PROPVARIANT &property = properties[i];
     switch(propIDs[i])
     {
       case NCoderPropID::kNumPasses:
