@@ -14,7 +14,7 @@ static const char kIllegalChar = '\0';
 static const char kNewLineChar = '\n';
 
 static const char *kEOFMessage = "Unexpected end of input stream";
-static const char *kReadErrorMessage  ="Error reading input stream"; 
+static const char *kReadErrorMessage  ="Error reading input stream";
 static const char *kIllegalCharMessage = "Illegal character in input stream";
 
 static LPCTSTR kFileOpenMode = TEXT("r");
@@ -31,7 +31,7 @@ bool CStdInStream::Open(LPCTSTR fileName)
 
 bool CStdInStream::Close()
 {
-  if(!_streamIsOpen)
+  if (!_streamIsOpen)
     return true;
   _streamIsOpen = (fclose(_stream) != 0);
   return !_streamIsOpen;
@@ -48,12 +48,12 @@ AString CStdInStream::ScanStringUntilNewLine()
   for (;;)
   {
     int intChar = GetChar();
-    if(intChar == EOF)
+    if (intChar == EOF)
       throw kEOFMessage;
     char c = char(intChar);
     if (c == kIllegalChar)
       throw kIllegalCharMessage;
-    if(c == kNewLineChar)
+    if (c == kNewLineChar)
       break;
     s += c;
   }
@@ -64,7 +64,7 @@ void CStdInStream::ReadToString(AString &resultString)
 {
   resultString.Empty();
   int c;
-  while((c = GetChar()) != EOF)
+  while ((c = GetChar()) != EOF)
     resultString += char(c);
 }
 
@@ -75,8 +75,8 @@ bool CStdInStream::Eof()
 
 int CStdInStream::GetChar()
 {
-  int c = fgetc(_stream);
-  if(c == EOF && !Eof())
+  int c = fgetc(_stream); // getc() doesn't work in BeOS?
+  if (c == EOF && !Eof())
     throw kReadErrorMessage;
   return c;
 }
